#ifndef BLIEPDSP_H
#define BLIEPDSP_H
#include <vcl.h>

/* PI is needed */
#define PI 3.141592653
/* Values converted from big endian, id = RIFF, format = WAVE */
#define PCM_RIFF 0x46464952
#define PCM_FORMAT 0x45564157
/* Recursive filter parameters */
#define HIGH_PASS 0xF0
#define LOW_PASS 0xF1
#define NARROW_PASS 0xF2
#define NARROW_REJECT 0xF3

class bliepdsp {
  public:
  /* Data */
  char *data;
  unsigned int num_samples;

  /* WAV header chunk */
  struct wavheader {
    unsigned int id;
    unsigned int size;
    unsigned int format;
    unsigned int sub_id;
    unsigned int sub_size;
    unsigned short int audio_format;
    unsigned short int num_channels;
    unsigned int sample_rate;
    unsigned int byte_rate;
    unsigned short int block_align;
    unsigned short int bits_per_sample;
    unsigned int data_id;
    unsigned int data_size;
  } header;

  /* Calculate play time of WAV file */
  unsigned int calculate_time (void);

  /* Return duration in format mm:ss */
  AnsiString get_duration (void);

  /* Read WAV file in buffer */
  void read_wav (const char *filename);

  /* Write out WAV file */
  void write_wav (const char *filename);

  /* Free the buffer */
  void freedata (void);

  /*
  Echo filter with programmable delay (ms) and programmable wet value (0-100%)
  8, 16 bit uncompressed PCM WAV
  1-65535 channel audio
  */
  void filter_echo (bliepdsp *out, const unsigned int delay, const unsigned int wet);

  /*
  Reverb filter with programmable delay, max impedance and programmable wet value.
  8, 16 bit uncompressed PCM WAV.
  -65535 channel audio.
  out Object containing the actual result of the reverb.
  Delay in milliseconds for the reverberation part.
  Wet value for the echo. More wet means more echo.
  minimp Percentual impedance value on where to stop the filter.
  */
  void filter_reverb (bliepdsp *out, const unsigned int delay, const unsigned int wet, const unsigned int minimp);

  /*
  Backwards filter, puts sound buffer in reverse
  8, 16, (24, 32)* bit PCM WAV
  1-65535 channel audio
   * untested
  */
  void filter_backwards (bliepdsp *out);

  /*
  Stretch filter, stretches the sound buffer without making the bitrate and sample
  rate change with programmable stretch (0-x%, 100% is current)
  8, 16, (24, 32)* bit PCM WAV
  1-65535 channel audio
   * untested
  */
  void filter_stretch (bliepdsp *out, const unsigned int stretch);

  /*
  Recursive (single pole) all-in-one filter.
  8, 16 bit PCM WAV
  1-65535 channel audio
  Currently supports:
  HIGH_PASS (co is amount of low suppressed, 1-90%)
  LOW_PASS (co is amount of high suppressed, 1-90%)
  NARROW_PASS (co is frequency to pass, Hz, max half the sample rate)
  NARROW_REJECT (co is frequency to reject, Hz, max half the sample rate)
  */
  void bliepdsp::filter_recursive (bliepdsp *out, const unsigned int filter, const unsigned int co);

  /*
  Copy sample from stream
  Parameters are multi-channel sample numbers
  8, 16, (24, 32)* bit PCM WAV
  1-65535 channel audio
   * untested
  */
  void bliepdsp::copy(bliepdsp *out, const unsigned int start, const unsigned int stop);

  /*
  Cut sample from stream
  Parameters are multi-channel sample numbers
  8, 16, (24, 32)* bit PCM WAV
  1-65535 channel audio
   * untested
  */
  void bliepdsp::cut(bliepdsp *out, const unsigned int start, const unsigned int stop);

  /*
  Paste sample into stream
  Parameter is a multi-channel sample number
  8, 16, (24, 32)* bit PCM WAV
  1-65535 channel audio
   * untested
  */
  void bliepdsp::paste(bliepdsp *in, const unsigned int where);

  /*
  Fade out the stream
  Fade time long (ms)
  8, 16  bit PCM WAV
  1-65535 channel audio
  */
  void bliepdsp::fade_out(bliepdsp *out, const unsigned int fadetime);

  /*
  Fade out the stream
  Fade time long (ms)
  8, 16  bit PCM WAV
  1-65535 channel audio
  */
  void bliepdsp::fade_in(bliepdsp *out, const unsigned int fadetime);
};
#endif
